#include <QtWidgets/QApplication>
#include <stdio.h>
#include <QDebug>
#include <QTcpSocket>
#include <QMessageBox>
#include <QtSerialPort/QtSerialPort>
#include <QtSerialPort/QSerialPortInfo>
#include <QCommandLineParser>

#include "microcontroller.h"
#include "trainjs.h"
#include "ui/mainwindow.h"
#include "items/itemstore.h"
#include "items/train.h"
#include "items/turnout.h"
#include "items/signal.h"

#define BAUD  QSerialPort::Baud38400

int main(int argc, char *argv[])
{
	QApplication a(argc, argv);

	//set info
	QCoreApplication::setOrganizationName("UVOS");
	QCoreApplication::setOrganizationDomain("uvos.xyz");
	QCoreApplication::setApplicationName("traincontrollerui");
	QCoreApplication::setApplicationVersion("0.1");

	QDir::setCurrent(a.applicationDirPath());

	//parse comand line
	QCommandLineParser parser;
	parser.setApplicationDescription("Smart Home Interface");
	parser.addHelpOption();
	parser.addVersionOption();
	QCommandLineOption tcpOption(QStringList() << "t" << "tcp", QCoreApplication::translate("main", "Use Tcp connection"));
	parser.addOption(tcpOption);
	QCommandLineOption hostOption(QStringList() << "H" << "host", QCoreApplication::translate("main",
	                              "Set server host ip addres"), "adress");
	parser.addOption(hostOption);
	QCommandLineOption portOption(QStringList() << "p" << "port", QCoreApplication::translate("main",
	                              "Set server Port in TCP mode or Serial port in serial mode"), "port");
	parser.addOption(portOption);
	QCommandLineOption serialOption(QStringList() << "s" << "serial", QCoreApplication::translate("main",
	                                "Use serial connection"));
	parser.addOption(serialOption);
	QCommandLineOption baudOption(QStringList() << "b" << "baud", QCoreApplication::translate("main", "Set Baud Rate"));
	parser.addOption(baudOption);
	parser.process(a);

	QIODevice* masterIODevice = nullptr;

	if(parser.isSet(tcpOption))
	{
		QTcpSocket* microSocket = new QTcpSocket;

		int port = 6856;
		if(parser.isSet(portOption)) port =  parser.value(portOption).toInt();

		QString host("127.0.0.1");
		if(parser.isSet(hostOption)) host = parser.value(hostOption);
		std::cout<<"connecting to "<<host.toStdString()<<':'<<port<<'\n';
		microSocket->connectToHost(host, port, QIODevice::ReadWrite);
		if(!microSocket->waitForConnected(3000))
		{
			std::cout<<"Can not connect to to Server.\n";
			QMessageBox::critical(nullptr, "Error", "Can not connect to to Server");
			return 1;
		}
		masterIODevice = microSocket;
	}
	else
	{
		QSerialPort* microPort = new QSerialPort;
		if(parser.isSet(portOption)) microPort->setPortName(parser.value(portOption));
		else microPort->setPortName("ttyUSB0");

		if(parser.isSet(portOption)) microPort->setBaudRate(parser.value(baudOption).toInt());
		else microPort->setBaudRate(BAUD);

		if(!microPort->open(QIODevice::ReadWrite))
		{
			QMessageBox::critical(nullptr, "Error", QString("Can not open serial port ")+microPort->portName());
			std::cout<<"Can not open serial port "<<microPort->portName().toStdString()<<". Continueing in demo mode"<<'\n';
			return 1;
		}
		masterIODevice = microPort;
	}

	Microcontroller micro(masterIODevice);
	micro.setPower(true);

	TrainJs::init();

	Train::micro = &micro;
	Turnout::micro = &micro;
	Signal::micro = &micro;
	ItemStore items;

	QObject::connect(&micro, &Microcontroller::gotItemList, &items, &ItemStore::addItems);
	QObject::connect(&micro, &Microcontroller::itemChanged, &items, &ItemStore::itemStateChanged);

	//mainwindow
	MainWindow w(&micro, &items);

	w.show();

	int retVal = a.exec();
	micro.setPower(false);

	if(masterIODevice)
		delete masterIODevice;
	return retVal;
}

